/*
 * $Id$
 * $Author$
 * $Date$
 *
 *
 * Taken from walnut.c
 *
 *    Copyright(c) 1999-2000 Grant Erickson <grant@lcse.umn.edu>
 *
 *    Copyright 2000-2002 MontaVista Software Inc.
 *      Completed implementation.
 *      Author: MontaVista Software, Inc.  <source@mvista.com>
 *
 *
 * Copyright 2005 Eric Gumtow, Network Appliance, Inc.
 * Copyright 2005 Dan Gilson, Network Appliance, Inc.
 *
 */

#include <linux/config.h>
#include <linux/init.h>
#include <linux/smp.h>
#include <linux/threads.h>
#include <linux/param.h>
#include <linux/string.h>
#include <linux/blk.h>
#include <linux/delay.h>
#include <linux/ibm_ocp_gpio.h>
#include <linux/mtd/physmap.h>

#include <asm/system.h>
#include <asm/processor.h>
#include <asm/machdep.h>
#include <asm/page.h>
#include <asm/time.h>
#include <asm/io.h>
#include <asm/todc.h>

#include "luna.h"
#include "luna_pvt.h"


extern void abort(void);


static u8 *cpld_reboot_info_reg = NULL;


luna_reboot_t
get_reboot_reason(void)
{
	u8 reboot_info = REBOOTED_VIA_UNKNOWN;
	u8 all_reboot_reasons = REBOOTED_VIA_POWER_ON |
	                        REBOOTED_VIA_WATCHDOG |
	                        REBOOTED_VIA_SW |
	                        REBOOTED_VIA_APLNCE;

	/* has board_io_mapping() been called? */
	if (cpld_reboot_info_reg) {
		reboot_info = *cpld_reboot_info_reg;
	}

	switch (reboot_info & (all_reboot_reasons)) {
		case REBOOTED_VIA_POWER_ON:
			return REBOOTED_VIA_POWER_ON;
		case REBOOTED_VIA_WATCHDOG:
			return REBOOTED_VIA_WATCHDOG;
		case REBOOTED_VIA_SW:
			return REBOOTED_VIA_SW;
		case REBOOTED_VIA_APLNCE:
			return REBOOTED_VIA_APLNCE;
		default:
			return REBOOTED_VIA_UNKNOWN;
	}

	return REBOOTED_VIA_UNKNOWN;
}

char *
reboot_reason_to_str(luna_reboot_t reboot_reason)
{
	switch (reboot_reason) {
		case REBOOTED_VIA_POWER_ON:
			return "power on";
		case REBOOTED_VIA_WATCHDOG:
			return "watchdog timeout";
		case REBOOTED_VIA_SW:
			return "software request";
		case REBOOTED_VIA_APLNCE:
			return "appliance request";
		default:
			return "unknown";
	}

	return "unknown";
}

luna_fw_img_t
get_fw_img_exec(void)
{
	u8 reboot_info = EXEC_UNKNOWN_FW;

	/* has board_io_mapping() been called? */
	if (cpld_reboot_info_reg) {
		reboot_info = *cpld_reboot_info_reg;
		if (reboot_info & EXEC_BACKUP_FW) {
			return EXEC_BACKUP_FW;
		}
		return EXEC_PRMRY_FW;
	}

	return EXEC_UNKNOWN_FW;
}

char *
fw_img_to_str(luna_fw_img_t fw_img)
{
	switch (fw_img) {
		case EXEC_BACKUP_FW:
			return "backup";
		case EXEC_PRMRY_FW:
			return "primary";
		default:
			return "unknown";
	}

	return "unknown";
}

void __init
board_setup_arch(void)
{
	/* board_setup_arch() is called AFTER board_init() */

	printk("Rebooted via %s.  Now executing %s firmware image.\n",
	 reboot_reason_to_str(get_reboot_reason()),
	 fw_img_to_str(get_fw_img_exec()));
	printk(KERN_INFO"\n");
}

void __init
board_io_mapping(void)
{
	/* CPLD needs just one byte mapped in */
	cpld_reboot_info_reg = ioremap(CPLD_PHYS_ADDR, 1);
	physmap_configure(0xff000000, 0x1000000, 2, NULL);
}

void __init
board_setup_irq(void)
{
	/* make your calls to request_irq() here 
	 *
	 * request_irq(IRQ_NUM, handler(int, void *, pt_regs *), 
	 *  flags, devname, devid)
	 */
}

void
luna_board_restart(char *cmd)
{
	int flash_idle, err, ms, tries;

	if (cmd) {
		printk("%s\n", cmd);
	}

	/* wait for the flash to become idle */
	ms = 0;
	do {
		err = ibm_gpio_kern_get(LUNA_GPIO_FLASH_BUSY, &flash_idle);
		if (err) {
			flash_idle = 0;
		}
		mdelay(MAX_UDELAY_MS);
		ms += MAX_UDELAY_MS;
	} while (ms < MAX_WAIT_FLASH_MS && !flash_idle);

	if (!flash_idle) {
		/* how do we want to handle this? */
		printk("flash device busy.\n");
	}

	/* clear the reset GPIO bit (signal the CPLD) */
	tries = 0;
	do {
		err = ibm_gpio_kern_clr(LUNA_GPIO_RESET_CPU);
		mdelay(1);
		tries++;
	} while (err && tries < MAX_GPIO_RESET_TRIES);

	if (err) {
		/* couldn't signal the CPLD so reset ourself */
		abort();
	}

	/* wait to be rebooted */
	ms = 0;
	while (ms < MAX_WAIT_BE_REBOOTED_MS) {
		mdelay(MAX_UDELAY_MS);
		ms += MAX_UDELAY_MS;
	}
	
	/* reboot never came so do it ourself */
	abort();
}

void __init
board_init(void)
{
	printk("NetApp Luna RLM Platform\n");

	/* do Luna-specific stuff here */
}
