/*
 * $Id$
 * $Author$
 * $Date$
 *
 * 
 * Taken from walnut.c
 *
 *    Copyright (c) 1999 Grant Erickson <grant@lcse.umn.edu>
 *
 *    Copyright 2000 MontaVista Software Inc.
 *	PPC405 modifications
 * 	Author: MontaVista Software, Inc.
 *         	frank_rowand@mvista.com or source@mvista.com
 * 	   	debbie_chu@mvista.com
 *
 *
 * Copyright 2005 Eric Gumtow, Network Appliance, Inc.
 * 
 */

#ifdef __KERNEL__
#ifndef __ASM_LUNA_H__
#define __ASM_LUNA_H__

/* We have a 405GP core */
#include <platforms/ibm405gp.h>

#ifndef __ASSEMBLY__
/*
 * Data structure defining board information maintained by the boot
 * ROM on IBM's "Walnut" evaluation board. An effort has been made to
 * keep the field names consistent with the 8xx 'bd_t' board info
 * structures.
 */

typedef struct board_info {
	unsigned char	 bi_s_version[4];	/* Version of this structure */
	unsigned char	 bi_r_version[30];	/* Version of the IBM ROM */
	unsigned int	 bi_memsize;		/* DRAM installed, in bytes */
	unsigned char	 bi_enetaddr[6];	/* Local Ethernet MAC address */
	unsigned char	 bi_pci_enetaddr[6];	/* PCI Ethernet MAC address */
	unsigned int	 bi_intfreq;		/* Processor speed, in Hz */
	unsigned int	 bi_busfreq;		/* PLB Bus speed, in Hz */
	unsigned int	 bi_pci_busfreq;	/* PCI Bus speed, in Hz */
	unsigned int	 bi_opb_busfreq;	/* OPB Bus speed, in Hz */
	int		 bi_iic_fast[1];	/* Use fast i2c mode */
} bd_t;

/* Some 4xx parts use a different timebase frequency from the internal clock.
*/
#define bi_tbfreq bi_intfreq


#ifdef CONFIG_PPC405GP_INTERNAL_CLOCK
#define BASE_BAUD		201600
#else
#define BASE_BAUD		691200
#endif

#define PPC4xx_MACHINE_NAME	"NetApp Luna"

void luna_board_restart(char *cmd);

typedef enum {
	REBOOTED_VIA_POWER_ON = 0x00,
	REBOOTED_VIA_WATCHDOG = 0x80,
	REBOOTED_VIA_SW = 0x40,
	REBOOTED_VIA_APLNCE = 0x20,
	REBOOTED_VIA_UNKNOWN = 0xff
} luna_reboot_t;

typedef enum {
	EXEC_PRMRY_FW = 0x00,
	EXEC_BACKUP_FW = 0x10,
	EXEC_UNKNOWN_FW = 0xff
} luna_fw_img_t;

luna_reboot_t get_reboot_reason(void);
char *reboot_reason_to_str(luna_reboot_t reboot_reason);
luna_fw_img_t get_fw_img_exec(void);
char *fw_img_to_str(luna_fw_img_t fw_img);

#endif /* !__ASSEMBLY__ */
#endif /* __ASM_LUNA_H__ */
#endif /* __KERNEL__ */
