/*
 * FILE NAME ibm_ocp_gpio.h
 *
 * BRIEF MODULE DESCRIPTION
 *	Generic gpio.
 *
 *  Armin Kuster akuster@pacbell.net
 *  Sept, 2001
 *
 *  Orignial driver
 *  Author: MontaVista Software, Inc.  <source@mvista.com>
 *          Frank Rowand <frank_rowand@mvista.com>
 *
 * Copyright 2000 MontaVista Software Inc.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  either version 2 of the  License, or (at your
 *  option) any later version.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Copyright 2005 Eric Gumtow, Network Appliance, Inc.
 */

#ifndef __IBM_OCP_GPIO_H
#define __IBM_OCP_GPIO_H

#include <linux/ioctl.h>
#include <linux/types.h>

#define IBMGPIO_IOCTL_BASE	'Z'

struct ibm_gpio_ioctl_data {
	__u32 device;
	__u32 mask;
	__u32 data;
};

#define GPIO_MINOR             185
#define IBMGPIO_IN		_IOWR(IBMGPIO_IOCTL_BASE, 0, struct ibm_gpio_ioctl_data)
#define IBMGPIO_OUT		_IOW (IBMGPIO_IOCTL_BASE, 1, struct ibm_gpio_ioctl_data)
#define IBMGPIO_OPEN_DRAIN	_IOW (IBMGPIO_IOCTL_BASE, 2, struct ibm_gpio_ioctl_data)
#define IBMGPIO_TRISTATE	_IOW (IBMGPIO_IOCTL_BASE, 3, struct ibm_gpio_ioctl_data)
#define IBMGPIO_CFG		_IOW (IBMGPIO_IOCTL_BASE, 4, struct ibm_gpio_ioctl_data)

typedef enum {
	LUNA_GPIO_TEST_MODE = 1,
	LUNA_GPIO_RESET_CPU,
	LUNA_GPIO_PRIMARY_IMG,
	LUNA_GPIO_STATUS_LED,
	LUNA_GPIO_POST_LED_8,
	LUNA_GPIO_BACKUP_IMG,
	LUNA_GPIO_WATCHDOG,
	LUNA_GPIO_RESERVED_1,
	LUNA_GPIO_RESERVED_2,
	LUNA_GPIO_RESERVED_3,
	LUNA_GPIO_POST_LED_1,
	LUNA_GPIO_POST_LED_7,
	LUNA_GPIO_POST_LED_6,
	LUNA_GPIO_POST_LED_5,
	LUNA_GPIO_POST_LED_4,
	LUNA_GPIO_POST_LED_3,
	LUNA_GPIO_IRQ_0,
	LUNA_GPIO_IRQ_1,
	LUNA_GPIO_IRQ_2,
	LUNA_GPIO_IRQ_3,
	LUNA_GPIO_FLASH_BUSY,
	LUNA_GPIO_POST_LED_2,
	LUNA_GPIO_RESERVED_4
} luna_gpio_t;
	
int ibm_gpio_kern_set(luna_gpio_t bit);
int ibm_gpio_kern_clr(luna_gpio_t bit);
int ibm_gpio_kern_get(luna_gpio_t bit, int *val);

#endif
